local L = AceLibrary("AceLocale-2.2"):new("DrDamage")

DrDamage.PlayerAura = {}
DrDamage.TargetAura = {}
DrDamage.Consumables = {}
DrDamage.Calculation = {}

local function DrD_LoadAuras()
	local playerClass = select(2,UnitClass("player"))
	local playerHealer = (playerClass == "PRIEST") or (playerClass == "SHAMAN") or (playerClass == "PALADIN") or (playerClass == "DRUID")
	local playerCaster = (playerClass == "MAGE") or (playerClass == "PRIEST") or (playerClass == "WARLOCK")
	local playerMelee = (playerClass == "ROGUE") or (playerClass == "WARRIOR") or (playerClass == "HUNTER")
	local playerHybrid = (playerClass == "DEATHKNIGHT") or (playerClass == "DRUID") or (playerClass == "PALADIN") or (playerClass == "SHAMAN")	
	local Aura = DrDamage.PlayerAura
	local GetSpellInfo = GetSpellInfo
	local arcane, fire, frost, nature, shadow, holy
	
	if playerCaster or playerHybrid then
		arcane = (playerClass == "DRUID") or (playerClass == "MAGE")
		fire = (playerClass == "SHAMAN") or (playerClass == "WARLOCK") or (playerClass == "MAGE")
		frost = (playerClass == "DEATHKNIGHT") or (playerClass == "SHAMAN") or (playerClass == "MAGE")
		nature = (playerClass == "SHAMAN") or (playerClass == "DRUID")
		shadow = (playerClass == "DEATHKNIGHT") or (playerClass == "PRIEST") or (playerClass == "WARLOCK")
		holy = (playerClass == "PRIEST") or (playerClass == "PALADIN")
	end
	
	--[[ NOTES: 
	 	School  = "All" means it applies to both heals and spells, not melee
		School = "Healing" applies only to heals
		School = "Physical" applies only to physical type abilities, which is the default in melee module unless otherwise specified
		No school or spell applies to everything but healing
		Caster = true or Melee = true limits it to respective modules
	--]]
	
	--Bloodlust
	Aura[GetSpellInfo(2825)] = { School = "All", Mods = { ["castTime"] = function(v) return v/1.3 end } }
	--Heroism
	Aura[GetSpellInfo(32182)] = Aura[GetSpellInfo(2825)]
	--Heroic Presence
	Aura[GetSpellInfo(28878)] = { Value = 1, ModType = "hitPerc", }
	
	if playerCaster or playerHybrid then
		--Totem of Wrath
		Aura[GetSpellInfo(30706)] = { School = "All", Mods = { ["spellDmg"] = 280 } }
		--Wrath of Air
		Aura[GetSpellInfo(3738)] = { Caster = true, School = "All", Mods = { ["castTime"] = function(v) return v/1.05 end } }
		--Power Infusion
		Aura[GetSpellInfo(10060)] = { Caster = true, School = "All", Mods = { ["castTime"] = function(v) return v/1.2 end } }		
		--Vibrant Blood
		Aura[GetSpellInfo(35329)] = { Caster = true, Value = 0.1 }
		--Bloodgem Infusion
		Aura[GetSpellInfo(34379)] = { Caster = true, School = "All", Value = 0.05, }
		--Moonkin Aura
		Aura[GetSpellInfo(24907)] = { Caster = true, School = "All", Mods = { ["critPerc"] = 5 } }
		--Elemental Oath
		Aura[GetSpellInfo(53414)] = { Caster = true, School = "All", Mods = { ["critPerc"] = 5 } }
		--Unrelenting Assault
		Aura[GetSpellInfo(64850)] = { Caster = true, School = "All", Value = -0.25, Ranks = 2, }
		--Hysteria (needs to be divided out from multiplier)
		Aura[GetSpellInfo(49016)] = { Caster = true, Value = 1 - 1/1.3, }		
	end
	if playerHealer then
		--A'dal's Song of Battle
		Aura[GetSpellInfo(39953)] = { School = "Healing", Value = 0.05 }
		--Vile Slime
		Aura[GetSpellInfo(40099)] = { School = "Healing", Value = -0.5 }
		--Debilitating Spray
		Aura[GetSpellInfo(40079)] = { School = "Healing", Value = -0.5 }
		--Vile Sludge
		Aura[GetSpellInfo(38246)] = { School = "Healing", Value = -0.5 }	
		--Nether Portal - Serenity
		Aura[GetSpellInfo(30422)] = { School = "Healing", Value = 0.05, Apps = 99 }
	end
	
	--Target debuffs
	Aura = DrDamage.TargetAura

	--Icebound Fortitude (How to implement this reduction properly?)
	Aura[GetSpellInfo(48792)] = { Value = -0.2 }
	--Bone Shield
	Aura[GetSpellInfo(49222)] = { Value = -0.2 }
	--Blade Barrier
	Aura[GetSpellInfo(51789)] = { Value = -0.01, Ranks = 5 }
	--Totem of Wrath
	Aura[GetSpellInfo(57722)] = { Value = 3, ModType = "critPerc" }
	--Heart of the Crusader
	Aura[GetSpellInfo(20335)] = { Value = 1, Ranks = 3, ModType = "critPerc" }
	--Master Poisoner
	Aura[GetSpellInfo(58410)] = { Value = 1, Ranks = 3, ModType = "critPerc" }
	--Focused Will
	Aura[GetSpellInfo(45234)] = { ModType = "Special" }
	local UnitExists = UnitExists
	local UnitIsFriend = UnitIsFriend
	DrDamage.Calculation[GetSpellInfo(45234)] = function( calculation, _, _, _, apps, _, rank )
		rank = rank and tonumber(string_match(rank,"%d+")) or 3
		apps = apps or 3
		if calculation.healingSpell then
			if not UnitExists("target") or UnitIsFriend("target","player") then
				calculation.dmgM = calculation.dmgM * (1 + (rank * 0.01 + 0.02) * apps)
			end
		elseif UnitExists("target") and not UnitIsFriend("target","player") then
			calculation.dmgM = calculation.dmgM * (1 - (rank * 0.01 + 0.01) * apps)
		end	
	end	
	
	
	if playerCaster or playerHybrid then
		--SCHOOL SPECIFIC
		--Shadow/Arcane/Fire/Frost/Nature Vulnerability
		if shadow or arcane or fire or frost or nature then 
			--Curse of the Elements
			Aura[GetSpellInfo(1490)] = { School = { ["Shadow"] = true, ["Arcane"] = true, ["Fire"] = true, ["Frost"] = true, ["Nature"] = true, ["Frostfire"] = true }, Ranks = 5, Value = { 0.06, 0.08, 0.1, 0.1, 0.13 }, ActiveAura = "Soul Siphon", }
			--Earth and Moon 
			Aura[GetSpellInfo(60431)] = { School = { ["Shadow"] = true, ["Arcane"] = true, ["Fire"] = true, ["Frost"] = true, ["Nature"] = true, ["Frostfire"] = true }, Ranks = 3, Value = { 0.04, 0.09, 0.13 }, }
		end
		
		--General	
		--Improved Shadow Bolt
		Aura[GetSpellInfo(17800)] = { Value = 1, Ranks = 5, ModType = "critPerc", Caster = true }
		--Misery
		Aura[GetSpellInfo(33196)] = { Value = 1, Ranks = 3, ModType = "hitPerc", Caster = true }		
		--Improved Scorch
		Aura[GetSpellInfo(22959)] = { Apps = 5, Value = 1, ModType = "critPerc", Caster = true }
		--Winter's Chill
		Aura[GetSpellInfo(12579)] = { Apps = 5, Value = 1, ModType = "critPerc", Caster = true }
		--Anti-Magic Shell
		Aura[GetSpellInfo(48707)] = { Value = -0.75, Caster = true }
		--Ebon Plague -- No easy way to tell what talent rank the Death Knight has without scanning the tooltip; for now assuming max rank.
		Aura[GetSpellInfo(51735)] = { Value = 0.13, Caster = true }
		
		--Weapon Debuffs
		--Spell Vulnerability (from Nightfall)
		Aura[GetSpellInfo(23605)] = { Value = 0.15, Caster = true }
		--Magic Disruption (Kael'thas fight Infinity Blade buff)
		Aura[GetSpellInfo(36478)] = { Value = 0.05, Apps = 5, Caster = true }
	end
	if playerHealer then
		--Amplify Magic
		Aura[GetSpellInfo(1008)] = { School = "Healing", ModType = "spellDmg", Ranks = 7, Value = { 16, 32, 53, 80, 96, 128, 255 }, }
		--Quick Recovery
		Aura[GetSpellInfo(31245)] = { School = "Healing", Value = 0.1, Ranks = 2, }
		--Demon Armor
		Aura[GetSpellInfo(27260)] = { School = "Healing", Value = 0.2, }
		--Curse of the Bleeding Hollow
		Aura[GetSpellInfo(34073)] = { School = "Healing", Value = -0.15 }
		--Wound Poison
		Aura[GetSpellInfo(43461)] = { School = "Healing", Value = -0.1, Apps = 5, }
		--Mortal Strike
		Aura[GetSpellInfo(12294)] = { School = "Healing", Value = -0.5, }
		--Veil of Shadow
		Aura[GetSpellInfo(17820)] = { School = "Healing", Value = -0.75, }
		--Nether Portal - Dominance
		Aura[GetSpellInfo(30423)] = { School = "Healing", Value = -0.01, Apps = 10 }
		--Divine Hymn
		Aura[GetSpellInfo(64844)] =  { School = "Healing", Value = 0.1 }
		--Guardian Spirit
		Aura[GetSpellInfo(47788)] =  { School = "Healing", Value = 0.4 }
		--Furious Attacks
		Aura[GetSpellInfo(46910)] = { School = "Healing", Value = -0.25, Apps = 2 }
		--Tree of Life
		Aura[GetSpellInfo(34123)] =  { School = "Healing", ModType = "Special" }
		local passive = select(2,GetSpellInfo(5420))
		DrDamage.Calculation[GetSpellInfo(34123)] = function( calculation, _, _, _, _, _, rank )
			if rank == passive then
				calculation.dmgM = calculation.dmgM * 1.06
			end
		end
		--Vampiric Blood
		Aura[GetSpellInfo(55233)] =  { School = "Healing", Value = 0.35 }
		
	end
	if playerMelee or playerHybrid then
		--Shadowform
		Aura[GetSpellInfo(15473)] = { School = { ["Ranged"] = true, ["Physical"] = true }, Value = -0.15 }
		--Holyform
		Aura[GetSpellInfo(46565)] = { School = { ["Ranged"] = true, ["Physical"] = true }, Value = -0.2 }
		--Armor Disruption
		Aura[GetSpellInfo(36482)] = { School = { ["Ranged"] = true, ["Physical"] = true }, Value = 0.05, Apps = 5 }
		--Hemorrhage
		Aura[GetSpellInfo(16511)] = { School = { ["Ranged"] = true, ["Physical"] = true }, Value = { 13, 21, 29, 42, 75 }, Ranks = 5, ModType = "dmgBonus", ActiveAura = "Hemorrhage" }
		--Bleed bonuses for Trauma, Mangle (Bear) and Mangle (Cat)
		Aura[GetSpellInfo(46856)] = { School = "Physical", Value = 0.15, Ranks = 2, ModType = "bleedBonus" }
		Aura[GetSpellInfo(33878)] = { School = "Physical", Value = 0.3, ModType = "bleedBonus" }
		Aura[GetSpellInfo(33876)] = { School = "Physical", Value = 0.3, ModType = "bleedBonus" }
		--Savage Combat
		Aura[GetSpellInfo(58683)] = { School = "Physical", Value = 0.02, Ranks = 2 }
		--Blood Frenzy
		Aura[GetSpellInfo(30070)] = { School = "Physical", Value = 0.02, Ranks = 2 }
	end
	
	local Consumables = DrDamage.Consumables
	if playerCaster or playerHybrid or playerHealer then
		--Flask of the Frost Wyrm
		Consumables[GetSpellInfo(53755)] = { School = "All", Mods = { ["spellDmg"] = 125 } }
		--Flask of Pure Mojo
		Consumables[GetSpellInfo(54212)] = { School = "All", Mods = { ["manaRegen"] = 7.6 } }
		--Spellpower Elixir
		Consumables[GetSpellInfo(33721)] = { School = "All", Mods = { ["spellDmg"] = 58 } }
		--Elixir of Mighty Mageblood
		Consumables[GetItemInfo(40109) or "Elixir of Mighty Mageblood"] = { School = "All", Mods = { ["manaRegen"] = 4.8 }, Alt = GetSpellInfo(53764) }
		--Food
		Consumables["+46 Spell Power Food"] = { Mods = { ["spellDmg"] = 46, }, Alt = GetSpellInfo(57327) }
		
	end
	if playerMelee or playerHybrid then
		--Flask of Endless Rage
		Consumables[GetSpellInfo(53760)] = { School = { ["Physical"] = true, ["Ranged"] = true }, Mods = { ["AP"] = 180 }, }
		--Elixir of Wrath
		Consumables[GetSpellInfo(53746)] = { School = { ["Physical"] = true, ["Ranged"] = true }, Mods = { ["AP"] = 90 }, }
		--Elixir of Demonslaying
		Consumables[GetSpellInfo(11406)] = { School = { ["Physical"] = true, ["Ranged"] = true }, Mods = { ["AP"] = 265 }, }
		--Food
		Consumables["+60 AP Food"] = { School = { ["Ranged"] = true, ["Physical"] = true }, Mods = { ["AP"] = 60 }, Alt = GetSpellInfo(57079) }
		Consumables["+80 AP Food"] = { School = { ["Ranged"] = true, ["Physical"] = true }, Mods = { ["AP"] = 80 }, Alt = GetSpellInfo(57079) }
	end
	
	--TODO: Fix hit rating to hit for casters
	--Elixir of Accuracy
	Consumables[GetItemInfo(44325) or "Elixir of Accuracy"] = { School = { ["All"] = true, ["Ranged"] = true, ["Physical"] = true }, Mods = { ["hitPerc"] = 1.372 }, Alt = GetSpellInfo(60340) }
	--Elixir of Deadly Strikes
	Consumables[GetItemInfo(44327) or "Elixir of Deadly Strikes"] = { School = { ["All"] = true, ["Ranged"] = true, ["Physical"] = true }, Mods = { ["critPerc"] = 0.9802, }, Alt = GetSpellInfo(60341) }
	--Food
	Consumables["+40 Hit Rating Food"] = { School = { ["All"] = true, ["Ranged"] = true, ["Physical"] = true }, Mods = { ["hitPerc"] = 1.22 }, Alt = GetSpellInfo(33263) }
	Consumables["+40 Critical Strike Rating Food"] = { School = { ["All"] = true, ["Ranged"] = true, ["Physical"] = true }, Mods = { ["critPerc"] = 0.871 }, Alt = GetSpellInfo(33263) }
end

DrD_LoadAuras()
DrD_LoadAuras = nil